@@ -13,6 +13,35 @@ using namespace arma;
 #include <algorithm>
 #include <cmath>
 
+#include <iostream>
+#include <random>
+#include <cmath>
+
+// Function to sample from an inverse Gaussian distribution
+double rinvgauss(double mu_tau, double lambda_tau, std::mt19937 &gen) {
+  // Step 1: Generate a random sample from the normal distribution
+  std::normal_distribution<double> rnorm(0.0, 1.0);
+  double z = rnorm(gen);
+  double z2 = z * z;
+  
+  // Step 2: Calculate xtau using the given formula
+  double xtau = mu_tau + 0.5 * mu_tau * mu_tau * z2 / lambda_tau -
+    0.5 * (mu_tau / lambda_tau) * std::sqrt(4 * mu_tau * lambda_tau * z2 + mu_tau * mu_tau * z2 * z2);
+  
+  // Step 3: Generate a random sample from the uniform distribution
+  std::uniform_real_distribution<double> runif(0.0, 1.0);
+  double u = runif(gen);
+  
+  // Step 4: Calculate tau based on the comparison
+  double tau = mu_tau * mu_tau / xtau;
+  if (u <= mu_tau / (mu_tau + xtau)) {
+    tau = xtau;
+  }
+  
+  return tau;
+}
+
+
 
 // [[Rcpp::export]]
 std::vector<std::vector<double>>  bayes(   std::vector<double> y,
@@ -32,21 +61,23 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
                                            bool updateE,
                                            bool updatePi,
                                            int nit,
-                                           int method) {
-  
-  
+                                           int nburn,
+                                           int nthin,
+                                           int method,
+                                           int seed) {
   
   // Define local variables
   int n = y.size();
   int m = W.size();
   int nc = pi.size();
+  double nsamples=0.0;
   
-    
   double rhs, lhs, bn, conv, diff, mu;
   double rhs0, rhs1, lhs0, lhs1, like0, like1, p0;
   double ssb, ssg, sse, dfb, dfe, dfg, chi2;
   double ssg_prior, nug;
-  double xtau, tau, lambda_tau, mu_tau, z, z2, u;
+  double lambda_tau, mu_tau, u, lambda2;
+  //double shape, shape0, rate, rate0, lambda2;
   
   std::vector<int> d(m);
   
@@ -55,8 +86,9 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   std::vector<int> order(m), mask(m);
   
   std::vector<double> dm(m),bm(m),vbi(m),vei(m);
-  std::vector<double> ves(nit),vbs(nit),vgs(nit),pis(nit),mus(nit);
-
+  //std::vector<double> ves(nit),vbs(nit),vgs(nit),pis(nit),mus(nit);
+  std::vector<double> ves(nit+nburn),vbs(nit+nburn),vgs(nit+nburn),pis(nit+nburn),mus(nit+nburn);
+  
   std::vector<double> vbscale(nc), pic(nc), pim(nc), probc(nc), logLc(nc);
   double cumprobc, vbc, v0, v1, logLcAdj;
   
@@ -82,7 +114,7 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   std::fill(pis.begin(), pis.end(), 0.0);
   std::fill(pim.begin(), pim.end(), 0.0);
   
-  
+
   // Initialize variables
   for (int i = 0; i < m; i++) {
     for (int j = 0; j < n; j++) {
@@ -97,6 +129,8 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   nug=nub;
   ssg_prior=((nug-2.0)/nug)*vg;
   
+  lambda2 = m;
+  
   // Establish order of markers as they are entered into the model
   std::iota(order.begin(), order.end(), 0);
   std::sort(  std::begin(order), 
@@ -106,13 +140,20 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   
   // Start Gibbs sampler
   std::random_device rd;
-  unsigned int local_seed;
-  local_seed = rd();
-  std::mt19937 gen(local_seed);
+  //unsigned int local_seed;
+  //local_seed = rd();
+  //std::mt19937 gen(local_seed);
+  std::mt19937 gen(seed);
   
-  for ( int it = 0; it < nit; it++) {
+  
+
+  for ( int it = 0; it < nit+nburn; it++) {
     conv = 0.0;
 
+    if ( (it > nburn) && (it % nthin == 0) ) {
+      nsamples = nsamples + 1.0;
+    }
+    
     // Compute marker effects (BLUP)
     if (method==0) {
       for ( int isort = 0; isort < m; isort++) {
@@ -179,10 +220,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
           e[j]=e[j] - W[i][j]*(diff);
         }
         b[i] = bn;
-        // ssb = b[i]*b[i];
-        // std::chi_squared_distribution<double> rchisq(dfb);
-        // chi2 = rchisq(gen);
-        // vbi[i] = (ssb + ssb_prior*nub)/chi2 ;
       }
     }
     
@@ -190,13 +227,16 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
     if ( method==3 ) {
       for ( int isort = 0; isort < m; isort++) {
         int i = order[isort];
-        lhs = ww[i] + lambda[i];
+        // Legarra et al 2011
+        lhs = ww[i]/ve + lambda[i];
+        // Park & Casella - Campos et al.
+        //lhs = ww[i]/ve + lambda[i]/ve;
         rhs = 0.0;
         for ( int j = 0; j < n; j++) {
-          rhs = rhs + W[i][j]*e[j]; 
+          rhs = rhs + W[i][j]*e[j];
         }
         rhs = rhs + ww[i]*b[i];
-        std::normal_distribution<double> rnorm(rhs/lhs, sqrt(ve/lhs));
+        std::normal_distribution<double> rnorm((rhs/ve)/lhs, sqrt(1.0/lhs));
         bn = rnorm(gen);
         diff = bn-b[i];
         for (int j = 0; j < n; j++) {
@@ -245,23 +285,24 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
 
       // Sample pi for Bayes C
       if(updatePi) {
-        dfb=0.0;
+        std::vector<double> mc(2);
+        std::fill(mc.begin(), mc.end(), 0.0);
         for (int i = 0; i<m ; i++) {
-          if(d[i]==1)   {
-            dfb = dfb + 1.0;
-          }
+          mc[d[i]] = mc[d[i]] + 1.0;
+        }
+        double pisum=0.0;
+        for (int j = 0; j<2 ; j++) {
+          std::gamma_distribution<double> rgamma(mc[j]+1.0,1.0);
+          double rg = rgamma(gen);
+          pi[j] = rg/m;
+          pisum = pisum + pi[j];
+        }
+        for (int j = 0; j<2 ; j++) {
+          pi[j] = pi[j]/pisum;
+          if(it>nburn) pim[j] = pim[j] + pi[j];
         }
-        double count = dfb + 1.0;
-        std::gamma_distribution<double> rgamma(count,1.0);
-        double rg = rgamma(gen);
-        pi[1] = rg/(double)m;
-        pi[0] = 1.0 - pi[1];
         pis[it] = pi[1];
-        pim[0] = pim[0] + pi[0];
-        pim[1] = pim[1] + pi[1];
       }  
-      
-            
     }
 
     // Sample marker effects (BayesR)
@@ -269,6 +310,7 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
       for ( int isort = 0; isort < m; isort++) {
         int i = order[isort];
         if(!mask[i])   continue;
+        
         // variance class likelihood version 1 
         rhs = 0.0;
         for ( int j = 0; j < n; j++) {
@@ -338,7 +380,7 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
         }
         for (int j = 0; j<nc ; j++) {
           pi[j] = pi[j]/pisum;
-          pim[j] = pim[j] + pi[j];
+          if(it>nburn) pim[j] = pim[j] + pi[j];
         }
       }
     }
@@ -350,29 +392,29 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
     dfb = 0.0;
     if (method<4) {
       for ( int i = 0; i < m; i++) {
-        bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         ssb = ssb + b[i]*b[i];
         dfb = dfb + 1.0;
-        dm[i] = dm[i] + 1.0;
+        if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
       }
     }
     if (method==4) {
       for ( int i = 0; i < m; i++) {
-        bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]==1)   {
           ssb = ssb + b[i]*b[i];
           dfb = dfb + 1.0;
-          dm[i] = dm[i] + 1.0;
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
         }
       }
     }
     if (method==5) {
       for ( int i = 0; i < m; i++) {
-        bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]>0)   {
           ssb = ssb + (b[i]*b[i])/gamma[d[i]];
           dfb = dfb + 1.0;
-          dm[i] = dm[i] + (double)d[i];
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + (double)d[i];
         }
       }
     }
@@ -392,6 +434,17 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
       for ( int j = 0; j < n; j++) {
         sse = sse + e[j]*e[j];
       }
+      // if(method==3) {
+      //   // Park & Casella - Campos et al.
+      //   ssb=0.0;
+      //   dfb=0.0;
+      //   for ( int i = 0; i < m; i++) {
+      //     ssb = ssb + b[i]*(1.0/lambda[i])*b[i]/ve;
+      //     dfb = dfb + 1.0;
+      //   }
+      //   sse=sse+ssb;
+      //   dfe=dfe+dfb;
+      // }
       std::chi_squared_distribution<double> rchisq(dfe);
       chi2 = rchisq(gen);
       ve = (sse + sse_prior*nue)/chi2 ;
@@ -400,20 +453,40 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
     
     // Sample marker specific tau for Bayes lasso
     if (method==3) { 
-      lambda_tau = 2.0*2.0*0.5*0.5/vb;
-      for ( int i = 0; i < m; i++) { 
-        ssb = b[i]*b[i];
-        mu_tau = sqrt(lambda_tau/ssb);
-        std::normal_distribution<double> rnorm(0.0, 1.0);
-        z = rnorm(gen);
-        z2=z*z;
-        xtau=mu_tau+0.5*mu_tau*mu_tau*z2/lambda_tau - 0.5*(mu_tau/lambda_tau)*sqrt(4*mu_tau*lambda_tau*z2+mu_tau*mu_tau*z2*z2);
-        std::uniform_real_distribution<double> runif(0.0, 1.0);
-        u = runif(gen);
-        tau = mu_tau*mu_tau/xtau;
-        if(u <= mu_tau/(mu_tau+xtau)) tau=xtau;
-        lambda[i] = ve/tau;
+      // Hyper parameters for the inverse Gaussian distribution
+      //double lrate0 = 0.1;  // lambda0 hyperparameter rate
+      double lshape0 = 1.0; // lambda0 hyperparameter shape
+      
+      // Set a small tolerance value (epsilon)
+      const double epsilon = 1e-8;
+      
+      lambda_tau = lambda2;
+      lambda_tau = std::max(lambda_tau, epsilon);
+      
+      // Compute mutau for each element in vector b
+      double sum_tau = 0.0;
+      for (size_t i = 0; i < b.size(); ++i) {
+        // If b[i] is too small (close to zero), use epsilon instead
+        double abs_b = std::abs(b[i]) < epsilon ? epsilon : std::abs(b[i]);
+        // Calculate mu_tau with the tolerance for very small b[i]
+        // Legarra et al 2011
+        mu_tau = std::sqrt(lambda_tau) / abs_b;
+        // Park & Casella - Campos et al.
+        //mu_tau = std::sqrt(lambda_tau*ve) / abs_b;
+        mu_tau = std::max(mu_tau, epsilon);
+        double invtau = rinvgauss(mu_tau, lambda_tau, gen);
+        lambda[i] = invtau;
+        sum_tau += 1.0/invtau;
       }
+      
+      // Calculate ratel2 and shl2
+      double shl2 = m + lshape0;    
+      //double ratel2 = sum_tau / 2.0 + lrate0;
+      //std::gamma_distribution<double> rgamma(shl2, 1.0/ratel2);
+      double scl2 = 2.0/sum_tau;
+      std::gamma_distribution<double> rgamma(shl2, scl2);
+      lambda2 = rgamma(gen);
+      lambda2 = std::max(1.0, std::min(lambda2, 1000000.0));
     }
     
     // Update mu and adjust residuals
@@ -437,52 +510,56 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
 
     
   }
-  
+
+
   // Summarize results
-  std::vector<std::vector<double>> result(11);
+  std::vector<std::vector<double>> result(12);
   result[0].resize(m);
   result[1].resize(m);
-  result[2].resize(nit);
-  result[3].resize(nit);
-  result[4].resize(nit);
-  result[5].resize(nit);
-  result[6].resize(nc);
-  result[7].resize(n);
-  result[8].resize(m);
+  result[2].resize(nit+nburn);
+  result[3].resize(nit+nburn);
+  result[4].resize(nit+nburn);
+  result[5].resize(nit+nburn);
+  result[6].resize(nit+nburn);
+  result[7].resize(nc);
+  result[8].resize(n);
   result[9].resize(m);
-  result[10].resize(3);
+  result[10].resize(m);
+  result[11].resize(3);
   
   for (int i=0; i < m; i++) {
-    result[0][i] = bm[i]/nit;
-    result[1][i] = dm[i]/nit;
+    //result[0][i] = bm[i]/nit;
+    //result[1][i] = dm[i]/nit;
+    result[0][i] = bm[i]/nsamples;
+    result[1][i] = dm[i]/nsamples;
   }
-  for (int i=0; i < nit; i++) {
+  for (int i=0; i < nit+nburn; i++) {
     result[2][i] = mus[i];
     result[3][i] = vbs[i];
     result[4][i] = vgs[i];
     result[5][i] = ves[i];
+    result[6][i] = pis[i];
   }
   for (int j=0; j < nc; j++) {
-    result[6][j] = pim[j]/nit;
+    result[7][j] = pim[j]/nit;
   }  
-  
   for (int i=0; i < n; i++) {
-    result[7][i] = y[i]- mu - e[i];
+    result[8][i] = y[i]- mu - e[i];
   }
   for (int i=0; i < m; i++) {
-    result[8][i] = b[i];
-    result[9][i] = d[i];
+    result[9][i] = b[i];
+    result[10][i] = d[i];
   }
-  result[10][0] = vb;
-  result[10][1] = ve;
-  result[10][2] = pi[0];
-  
+  result[11][0] = vb;
+  result[11][1] = ve;
+  result[11][2] = pi[0];
   return result;
-  
-
 }
 
 
+
+
+
 // [[Rcpp::export]]
 std::vector<std::vector<double>>  sbayes( std::vector<double> wy,
                                           std::vector<std::vector<double>> LD,
@@ -775,6 +852,7 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
                                               int n, 
                                               int nit,
                                               int nburn,
+                                              int nthin,
                                               int method,
                                               int algo,
                                               int seed) {
@@ -784,9 +862,12 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
   // Define local variables
   int m = b.size();
   int nc = pi.size();
+  double nsamples=0.0;
+  
   
   double rhs, lhs, bn, bj, diff;
   double rhs1, lhs1, like0, like1, p0, v0, v1;
+  //double rhs0, lhs0;
   double ssb, sse, ssg, dfb, dfe, dfg, chi2;
   double x_tau, tau, lambda_tau, mu_tau, z, z2, u, vbin;
   double shape, shape0, rate, rate0, lambda2;
@@ -877,7 +958,12 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
   std::mt19937 gen(seed);
   
   for ( int it = 0; it < nit+nburn; it++) {
-    
+  
+  if ( (it > nburn) && (it % nthin == 0) ) {
+    nsamples = nsamples + 1.0;
+  }
+  
+  
     // Compute marker effects (BLUP)
     if (method==0) {
       for ( int isort = 0; isort < m; isort++) {
@@ -1003,8 +1089,8 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
         //rhs1 = r[i] + ww[i]*b[i];
         //lhs0 = 1.0/vb;
         //lhs1 = ww[i]/vei[i] + 1.0/vb;
-        //like0 = std::log(1.0/std::sqrt(lhs0)) + 0.5*(rhs0*rhs0)/lhs0 + std::log(1.0-pi); 
-        //like1 = std::log(1.0/std::sqrt(lhs1)) + 0.5*(rhs1*rhs1)/lhs1 + std::log(pi); 
+        //like0 = std::log(1.0/std::sqrt(lhs0)) + 0.5*(rhs0*rhs0)/lhs0 + std::log(pi[0]); 
+        //like1 = std::log(1.0/std::sqrt(lhs1)) + 0.5*(rhs1*rhs1)/lhs1 + std::log(pi[1]); 
         //p0 = 1.0/(std::exp(like1 - like0) + 1.0);
         // version 2
         rhs = r[i] + ww[i]*b[i];
@@ -1038,20 +1124,41 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
       
       // Sample pi for Bayes C
       if(updatePi) {
-        dfb=0.0;
+        std::vector<double> mc(2);
+        std::fill(mc.begin(), mc.end(), 0.0);
         for (int i = 0; i<m ; i++) {
-          if(d[i]==1)   {
-            dfb = dfb + 1.0;
-          }
+          mc[d[i]] = mc[d[i]] + 1.0;
+        }
+        double pisum=0.0;
+        for (int j = 0; j<2 ; j++) {
+          std::gamma_distribution<double> rgamma(mc[j]+1.0,1.0);
+          double rg = rgamma(gen);
+          pi[j] = rg/m;
+          pisum = pisum + pi[j];
+        }
+        for (int j = 0; j<2 ; j++) {
+          pi[j] = pi[j]/pisum;
+          if(it>nburn) pim[j] = pim[j] + pi[j];
         }
-        double count = dfb + 1.0;
-        std::gamma_distribution<double> rgamma(count,1.0);
-        double rg = rgamma(gen);
-        pi[1] = rg/(double)m;
-        pi[0] = 1.0 - pi[1];
         pis[it] = pi[1];
-        if(it>nburn) pim[0] = pim[0] + pi[0];
-        if(it>nburn) pim[1] = pim[1] + pi[1];
+        // dfb=0.0;
+        // for (int i = 0; i<m ; i++) {
+        //   if(d[i]==1)   {
+        //     dfb = dfb + 1.0;
+        //   }
+        // }
+        // double count = dfb + 1.0;
+        // std::gamma_distribution<double> rgamma(count,1.0);
+        // double rg = rgamma(gen);
+        // double pisum=0.0;
+        // pi[1] = rg/(double)m;
+        // pi[0] = 1.0 - pi[1];
+        // pisum = pi[0] + pi[1];
+        // pi[0] = pi[0]/pisum;
+        // pi[1] = pi[1]/pisum;
+        // pis[it] = pi[1];
+        // if(it>nburn) pim[0] = pim[0] + pi[0];
+        // if(it>nburn) pim[1] = pim[1] + pi[1];
       }
       
     }
@@ -1146,29 +1253,30 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
     dfb = 0.0;
     if (method<4) {
       for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         ssb = ssb + b[i]*b[i];
         dfb = dfb + 1.0;
-        if(it>nburn) dm[i] = dm[i] + 1.0;
+        if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
       }
     }
     if (method==4) {
       for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]==1)   {
           ssb = ssb + b[i]*b[i];
           dfb = dfb + 1.0;
-          if(it>nburn) dm[i] = dm[i] + 1.0;
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
         }
       }
     }
     if (method==5) {
       for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]>0)   {
           ssb = ssb + (b[i]*b[i])/gamma[d[i]];
           dfb = dfb + 1.0;
-          if(it>nburn) dm[i] = dm[i] + (double)d[i];
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
+          //if(it>nburn) dm[i] = dm[i] + (double)d[i];
         }
       }
     }
@@ -1237,8 +1345,10 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
   result[10].resize(3);
 
   for (int i=0; i < m; i++) {
-    result[0][i] = bm[i]/nit;
-    result[1][i] = dm[i]/nit;
+    //result[0][i] = bm[i]/nit;
+    //result[1][i] = dm[i]/nit;
+    result[0][i] = bm[i]/nsamples;
+    result[1][i] = dm[i]/nsamples;
   }
   for (int i=0; i < nit+nburn; i++) {
     //result[2][i] = mus[i];
@@ -1267,60 +1377,62 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
 
 
 // [[Rcpp::export]]
-std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
-                                              std::vector<double> ww, 
-                                              std::vector<std::vector<double>> LDvalues, 
-                                              std::vector<std::vector<int>> LDindices, 
+std::vector<std::vector<double>>  sbayes_reg( std::vector<double>& wy,
+                                              std::vector<double>& ww, 
+                                              std::vector<std::vector<double>>& LDvalues, 
+                                              std::vector<std::vector<int>>& LDindices, 
                                               std::vector<double> b, 
                                               std::vector<double> lambda, 
                                               std::vector<bool> mask, 
                                               double yy, 
                                               std::vector<double> pi, 
                                               std::vector<double> gamma, 
-                                              double vg, 
                                               double vb, 
+                                              double vg, 
                                               double ve,
                                               double ssb_prior,
+                                              double ssg_prior,
                                               double sse_prior,
                                               double nub,
+                                              double nug,
                                               double nue,
                                               bool updateB,
                                               bool updateE,
                                               bool updatePi,
                                               bool updateG,
-                                              bool adjustE,
                                               int n, 
                                               int nit,
                                               int nburn,
+                                              int nthin,
                                               int method,
-                                              int algo) {
+                                              int algo,
+                                              int seed) {
   
   // Define local variables
   int m = b.size();
   int nc = pi.size();
+  double nsamples=0.0;
   
-  double rhs, lhs, bn, bj, diff;
+  double rhs, lhs, bn, diff, logcpo;
   double rhs1, lhs1, like0, like1, p0, v0, v1;
   double ssb, sse, ssg, dfb, dfe, dfg, chi2;
-  //double x_tau, tau, lambda_tau, mu_tau, z, z2, u, vbin;
-  //double shape, shape0, rate, rate0, lambda2;
-  double u, lambda2;
+  double u;
   
   std::vector<double> vbscale(nc), probc(nc), logLc(nc), pim(nc);
   double cumprobc, vbc, logLcAdj;
   
-  std::vector<int> d(m), order(m);
-  std::vector<double> r(m),vei(m);
+  std::vector<int> d(m),order(m);
+  std::vector<double> r(m),vei(m),x2(m),vadj(m);
   std::vector<double> dm(m),bm(m);
-  std::vector<double> x2(m),vadj(m),vbi(m);
-
+  std::vector<double> logsum(m);
+  
   std::vector<double> ves(nit+nburn),vbs(nit+nburn),vgs(nit+nburn),pis(nit+nburn);
   std::vector<std::vector<double>> bs(nit+nburn, std::vector<double>(m, 0.0));  
   std::vector<std::vector<int>> ds(nit+nburn, std::vector<int>(m, 0));  
+  std::vector<std::vector<double>> prob(nit+nburn, std::vector<double>(m, 0.0));  
   
   // Initialize variables
   for ( int i = 0; i < m; i++) {
-    vbi[i]=vb;
     r[i] = wy[i];
     x2[i] = (wy[i]/ww[i])*(wy[i]/ww[i]);
   }
@@ -1332,28 +1444,15 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
   std::fill(ves.begin(), ves.end(), 0.0);
   std::fill(pis.begin(), pis.end(), 0.0);
   std::fill(pim.begin(), pim.end(), 0.0);
+  std::fill(logsum.begin(), logsum.end(), 0.0);
   
-  // adjust sparseld
+  // Adjust residual
   for ( int i = 0; i < m; i++) {
     vadj[i] = 0.0;
-    if(adjustE) {
-      vadj[i] = ((double)m-(double)LDindices[i].size())/(double)m;
-    }  
+    vadj[i] = ((double)m-(double)LDindices[i].size())/(double)m;
     vei[i] = vadj[i]*vg + ve;
   }
   
-  // initialize BayesL parameters
-  dfb = (nub - 2.0)/nub;
-  lambda2 = 2.0*(1.0 - dfb)/(dfb)*n;
-  //shape0 = 1.1;
-  //rate0 = (shape0 - 1.0) / lambda2;
-  for ( int i = 0; i < m; i++) {
-    lambda[i] = sqrt(lambda2); 
-  }
-  for ( int i = 0; i < nc; i++) {
-    vbscale[i]=gamma[i];
-  }
-  
   // Establish order of markers as they are entered into the model
   std::iota(order.begin(), order.end(), 0);
   std::sort(  std::begin(order), 
@@ -1369,13 +1468,14 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
   
   // Start Gibbs sampler
   std::random_device rd;
-  unsigned int local_seed;
-  local_seed = rd();
-  std::mt19937 gen(local_seed);
-  
+  std::mt19937 gen(seed);
   
   for ( int it = 0; it < nit+nburn; it++) {
     
+    if ( (it > nburn) && (it % nthin == 0) ) {
+      nsamples = nsamples + 1.0;
+    }
+    
     // Sample marker effects (BayesC)
     if (method==4) {
       for ( int isort = 0; isort < m; isort++) {
@@ -1397,9 +1497,6 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
           lhs1 = ww[i] + vei[i]/vb;
           std::normal_distribution<double> rnorm(rhs1/lhs1, sqrt(vei[i]/lhs1));
           bn = rnorm(gen);
-          if(algo==2 && it<nburn ) {
-            bn = (1.0-p0)*bn;
-          }
         } 
         diff = (bn-b[i]);
         if(diff!=0.0) {
@@ -1412,20 +1509,23 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
       
       // Sample pi for Bayes C
       if(updatePi) {
-        dfb=0.0;
+        std::vector<double> mc(2);
+        std::fill(mc.begin(), mc.end(), 0.0);
         for (int i = 0; i<m ; i++) {
-          if(d[i]==1)   {
-            dfb = dfb + 1.0;
-          }
+          mc[d[i]] = mc[d[i]] + 1.0;
+        }
+        double pisum=0.0;
+        for (int j = 0; j<2 ; j++) {
+          std::gamma_distribution<double> rgamma(mc[j]+1.0,1.0);
+          double rg = rgamma(gen);
+          pi[j] = rg/m;
+          pisum = pisum + pi[j];
+        }
+        for (int j = 0; j<2 ; j++) {
+          pi[j] = pi[j]/pisum;
+          if(it>nburn && (it % nthin == 0)) pim[j] = pim[j] + pi[j];
         }
-        double count = dfb + 1.0;
-        std::gamma_distribution<double> rgamma(count,1.0);
-        double rg = rgamma(gen);
-        pi[1] = rg/(double)m;
-        pi[0] = 1.0 - pi[1];
         pis[it] = pi[1];
-        if(it>nburn) pim[0] = pim[0] + pi[0];
-        if(it>nburn) pim[1] = pim[1] + pi[1];
       }
       
     }
@@ -1472,16 +1572,6 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
           lhs =ww[i]+vei[i]/vbc;
           std::normal_distribution<double> rnorm(rhs/lhs, sqrt(vei[i]/lhs));
           bn = rnorm(gen);
-          if(algo==2 && it<nburn) {
-            bn=0.0;
-            for (size_t j = 1; j < gamma.size(); j++) {
-              vbc = vb * gamma[j];
-              lhs =ww[i]+vei[i]/vbc;
-              std::normal_distribution<double> rnorm(rhs/lhs, sqrt(vei[i]/lhs));
-              bj = rnorm(gen);
-              bn += probc[j]*bj;
-            }
-          }
         }
         diff = (bn-b[i]);
         if(diff!=0.0) {
@@ -1507,7 +1597,7 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
         }
         for (int j = 0; j<nc ; j++) {
           pi[j] = pi[j]/pisum;
-          if(it>nburn) pim[j] = pim[j] + pi[j];
+          if(it>nburn && (it % nthin == 0)) pim[j] = pim[j] + pi[j];
         }
         pis[it] = 1.0 - pi[0];
       }
@@ -1516,31 +1606,23 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
     // Sample marker variance
     ssb = 0.0;
     dfb = 0.0;
-    if (method<4) {
-      for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
-        ssb = ssb + b[i]*b[i];
-        dfb = dfb + 1.0;
-        if(it>nburn) dm[i] = dm[i] + 1.0;
-      }
-    }
     if (method==4) {
       for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]==1)   {
           ssb = ssb + b[i]*b[i];
           dfb = dfb + 1.0;
-          if(it>nburn) dm[i] = dm[i] + 1.0;
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
         }
       }
     }
     if (method==5) {
       for ( int i = 0; i < m; i++) {
-        if(it>nburn) bm[i] = bm[i] + b[i];
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
         if(d[i]>0)   {
           ssb = ssb + (b[i]*b[i])/gamma[d[i]];
           dfb = dfb + 1.0;
-          if(it>nburn) dm[i] = dm[i] + 1.0;
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
         }
       }
     }
@@ -1581,20 +1663,32 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
     if(updateG) {
       vg = ssg/dfg;
     }
-    if(adjustE) {
-      for ( int i = 0; i < m; i++) {
-        vei[i] = vadj[i]*vg + ve;
-      }
+    for ( int i = 0; i < m; i++) {
+      vei[i] = vadj[i]*vg + ve;
     }
     
+    // Compute logcpo
+    if(it>nburn && (it % nthin == 0)) {
+      double constant = 1.0 / std::sqrt(2 * M_PI);  
+      for (int i = 0; i < m; i++) {
+        logsum[i] += 1.0/(constant * std::exp(-0.5 * r[i] * r[i]));
+      }
+    }
+  
     for ( int i = 0; i < m; i++) {
       bs[it][i] = b[i];
       ds[it][i] = d[i];
     }
   }
+
+  // Compute logcpo
+  logcpo=0.0;
+  for (int i=0; i < m; i++) {
+    logcpo += std::log(nsamples*(1/logsum[i]));
+  }
   
   // Summarize results
-  std::vector<std::vector<double>> result(13);
+  std::vector<std::vector<double>> result(14);
   
   result[0].resize(m);
   result[1].resize(m);
@@ -1606,15 +1700,15 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
   result[7].resize(nc);
   result[8].resize(m);
   result[9].resize(m);
-  result[10].resize(3);
+  result[10].resize(4);
   result[11].resize((nit+nburn)*m);
   result[12].resize((nit+nburn)*m);
+  result[13].resize((nit+nburn)*m);
   for (int i=0; i < m; i++) {
-    result[0][i] = bm[i]/nit;
-    result[1][i] = dm[i]/nit;
+    result[0][i] = bm[i]/nsamples;
+    result[1][i] = dm[i]/nsamples;
   }
   for (int i=0; i < nit+nburn; i++) {
-    //result[2][i] = mus[i];
     result[2][i] = 0.0;
     result[3][i] = vbs[i];
     result[4][i] = vgs[i];
@@ -1622,22 +1716,299 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double> wy,
     result[6][i] = pis[i];
   }
   for (int j=0; j < nc; j++) {
-    result[7][j] = pim[j]/nit;
+    result[7][j] = pim[j]/nsamples;
   }  
   
   for (int i=0; i < m; i++) {
     result[8][i] = r[i];
     result[9][i] = b[i];
-    //result[9][i] = d[i];
   }
   result[10][0] = vb;
   result[10][1] = ve;
   result[10][2] = pi[0];
-
+  result[10][3] = logcpo;
+  
   for ( int it = 0; it < nit+nburn; it++) {
     for ( int i = 0; i < m; i++) {
       result[11][it*m + i] = bs[it][i];
       result[12][it*m + i] = (double)ds[it][i];
+      result[13][it*m + i] = prob[it][i];
+    }
+  }
+  return result;
+}
+
+
+
+// [[Rcpp::export]]
+std::vector<std::vector<double>>  sbayes_reg_eigen( std::vector<double>& wy,
+                                                    std::vector<double>& ww, 
+                                                    std::vector<std::vector<double>>& LDvalues, 
+                                                    std::vector<std::vector<int>>& LDindices, 
+                                                    std::vector<double> b, 
+                                                    std::vector<double> lambda, 
+                                                    std::vector<bool> mask, 
+                                                    std::vector<double> pi, 
+                                                    std::vector<double> gamma, 
+                                                    double vb, 
+                                                    double vg, 
+                                                    double ve,
+                                                    double ssb_prior,
+                                                    double ssg_prior,
+                                                    double sse_prior,
+                                                    double nub,
+                                                    double nug,
+                                                    double nue,
+                                                    bool updateB,
+                                                    bool updateE,
+                                                    bool updatePi,
+                                                    bool updateG,
+                                                    int n, 
+                                                    int nit,
+                                                    int nburn,
+                                                    int nthin,
+                                                    int method,
+                                                    int algo,
+                                                    int seed) {
+  
+  // Define local variables
+  int m = b.size();
+  int nc = pi.size();
+  double nsamples=0.0;
+  int q=wy.size();
+  
+  double rhs, lhs, bn, diff;
+  double ssb, sse, ssg, dfb, dfe, chi2;
+  double u;
+  
+  std::vector<double> probc(nc), logLc(nc), pim(nc);
+  double cumprobc, vbc, logLcAdj, logcpo;
+  
+  std::vector<int> d(m);
+  std::vector<double> r(m);
+  std::vector<double> dm(m),bm(m);
+  std::vector<double> logsum(q);
+  
+  std::vector<double> ves(nit+nburn),vbs(nit+nburn),vgs(nit+nburn),pis(nit+nburn);
+  std::vector<std::vector<double>> bs(nit+nburn, std::vector<double>(m, 0.0));  
+  std::vector<std::vector<int>> ds(nit+nburn, std::vector<int>(m, 0));  
+  std::vector<std::vector<double>> prob(nit+nburn, std::vector<double>(m, 0.0));  
+  
+  // Initialize variables
+  
+  std::fill(bm.begin(), bm.end(), 0.0);
+  std::fill(dm.begin(), dm.end(), 0.0);
+  std::fill(vbs.begin(), vbs.end(), 0.0);
+  std::fill(vgs.begin(), vgs.end(), 0.0);
+  std::fill(ves.begin(), ves.end(), 0.0);
+  std::fill(pis.begin(), pis.end(), 0.0);
+  std::fill(pim.begin(), pim.end(), 0.0);
+  std::fill(logsum.begin(), logsum.end(), 0.0);
+  
+  for ( int i = 0; i < q; i++) {
+    r[i] = wy[i];
+  }
+  
+  
+  // Start Gibbs sampler
+  std::random_device rd;
+  std::mt19937 gen(seed);
+  
+  
+  for ( int it = 0; it < nit+nburn; it++) {
+    
+    if ( (it > nburn) && (it % nthin == 0) ) {
+      nsamples = nsamples + 1.0;
+    }
+    
+    
+    // Sample marker effects (BayesR)
+    if (method == 4 || method == 5) {
+      for ( int i = 0; i < m; i++) {
+        if(mask[i])   continue;
+        
+        // variance class likelihood 
+        rhs=0.0;
+        for (size_t j = 0; j < LDvalues[i].size(); j++) {
+          rhs +=r[LDindices[i][j]]*LDvalues[i][LDindices[i][j]];
+        }
+        rhs = (rhs + b[i])*(ww[i]/ve);
+        logLc[0] = std::log(pi[0]);
+        for (int j = 1; j<nc ; j++) {
+          vbc = vb*gamma[j];
+          lhs = (ww[i] / ve) + (1 / vbc);
+          logLc[j] = 0.5*( (std::log(1.0/lhs)-std::log(vbc)) + (rhs*rhs)/lhs) + std::log(pi[j]); 
+        }
+        // variance class probability 
+        std::fill(probc.begin(), probc.end(), 0.0);
+        for (int j = 0; j<nc ; j++) {
+          logLcAdj = 0.0;
+          for (int k = 0; k<nc ; k++) {
+            logLcAdj += std::exp(logLc[k] - logLc[j]);
+          }
+          probc[j] = 1.0/logLcAdj;
+        }
+        prob[it][i] = 1.0 - probc[0];
+        // sample variance class indicator
+        std::uniform_real_distribution<double> runif(0.0, 1.0);
+        u = runif(gen);
+        d[i]=0;
+        cumprobc = 0.0;
+        for (int j = 0; j<nc ; j++) {
+          cumprobc += probc[j];
+          if(u < cumprobc){
+            d[i] = j;
+            break;
+          }
+        }
+        
+        // sample marker effect
+        bn=0.0;
+        if(d[i]>0) {
+          vbc = vb * gamma[d[i]];
+          lhs = (ww[i] / ve) + (1 / vbc);
+          std::normal_distribution<double> rnorm(rhs/lhs, sqrt(1.0/lhs));
+          bn = rnorm(gen);
+        }
+        diff = (bn-b[i]);
+        if(diff!=0.0) {
+          for (size_t j = 0; j < LDindices[i].size(); j++) {
+            r[LDindices[i][j]] += -LDvalues[i][LDindices[i][j]]*diff;
+          }
+        }
+        b[i] = bn;
+      }
+      
+      // Sample pi for Bayes R
+      if(updatePi) {
+        std::vector<double> mc(nc);
+        std::fill(mc.begin(), mc.end(), 0.0);
+        for (int i = 0; i<m ; i++) {
+          mc[d[i]] = mc[d[i]] + 1.0;
+        }
+        double pisum=0.0;
+        for (int j = 0; j<nc ; j++) {
+          std::gamma_distribution<double> rgamma(mc[j]+1.0,1.0);
+          double rg = rgamma(gen);
+          pi[j] = rg/m;
+          pisum = pisum + pi[j];
+        }
+        for (int j = 0; j<nc ; j++) {
+          pi[j] = pi[j]/pisum;
+          if(it>nburn && (it % nthin == 0)) pim[j] = pim[j] + pi[j];
+        }
+        pis[it] = 1.0 - pi[0];
+      }
+    }
+    
+    // Sample marker variance
+    ssb = 0.0;
+    dfb = 0.0;
+    if (method == 4 || method == 5) {
+      for ( int i = 0; i < m; i++) {
+        if(it>nburn && (it % nthin == 0)) bm[i] = bm[i] + b[i];
+        if(d[i]>0)   {
+          ssb += (b[i]*b[i])/gamma[d[i]];
+          dfb += 1.0;
+          if(it>nburn && (it % nthin == 0)) dm[i] = dm[i] + 1.0;
+        }
+      }
+    }
+    std::chi_squared_distribution<double> rchisq_vb(dfb+nub);
+    chi2 = rchisq_vb(gen);
+    vbs[it] = (ssb + ssb_prior*nub)/chi2 ;
+    if(updateB) vb = vbs[it];
+    
+    // Sample residual variance
+    sse = 0.0;
+    for ( int i = 0; i < q; i++) {
+      sse += r[i]*r[i];
+    }
+    dfe = (double)q + nue;
+    sse= sse*dfe;
+    std::chi_squared_distribution<double> rchisq_ve(dfe);
+    chi2 = rchisq_ve(gen);
+    ves[it] = (sse + sse_prior*nue)/chi2;
+    if(updateE) ve = ves[it];
+    
+    // Sample genetic variance
+    ssg = 0.0;
+    for (int i = 0; i < q; i++) {
+      ssg += (wy[i] - r[i]) * (wy[i] - r[i]);
+    }
+    vgs[it] = ssg;
+    if(updateG) vg = ssg;
+    
+    // Compute logcpo
+    if(it>nburn && (it % nthin == 0)) {
+      double constant = 1.0 / std::sqrt(2 * M_PI);  
+      for (int i = 0; i < q; i++) {
+        logsum[i] += 1.0/(constant * std::exp(-0.5 * r[i] * r[i]));
+      }
+    }
+
+    for ( int i = 0; i < m; i++) {
+      bs[it][i] = b[i];
+      ds[it][i] = d[i];
+    }
+  }
+  
+  // Compute logcpo
+  logcpo=0.0;
+  for (int i=0; i < q; i++) {
+    logcpo += std::log(nsamples*(1/logsum[i]));
+  }
+  
+  
+  // Summarize results
+  std::vector<std::vector<double>> result(14);
+  
+  result[0].resize(m);
+  result[1].resize(m);
+  result[2].resize(nit+nburn);
+  result[3].resize(nit+nburn);
+  result[4].resize(nit+nburn);
+  result[5].resize(nit+nburn);
+  result[6].resize(nit+nburn);
+  result[7].resize(nc);
+  result[8].resize(q);
+  result[9].resize(m);
+  result[10].resize(4);
+  result[11].resize((nit+nburn)*m);
+  result[12].resize((nit+nburn)*m);
+  result[13].resize((nit+nburn)*m);
+  for (int i=0; i < m; i++) {
+    result[0][i] = bm[i]/nsamples;
+    result[1][i] = dm[i]/nsamples;
+  }
+  for (int i=0; i < nit+nburn; i++) {
+    result[2][i] = 0.0;
+    result[3][i] = vbs[i];
+    result[4][i] = vgs[i];
+    result[5][i] = ves[i];
+    result[6][i] = pis[i];
+  }
+  for (int j=0; j < nc; j++) {
+    result[7][j] = pim[j]/nsamples;
+  }  
+  
+  for (int i=0; i < q; i++) {
+    result[8][i] = r[i];
+  }
+  
+  for (int i=0; i < m; i++) {
+    result[9][i] = b[i];
+  }
+  result[10][0] = vb;
+  result[10][1] = ve;
+  result[10][2] = pi[0];
+  result[10][3] = logcpo;
+  
+  for ( int it = 0; it < nit+nburn; it++) {
+    for ( int i = 0; i < m; i++) {
+      result[11][it*m + i] = bs[it][i];
+      result[12][it*m + i] = (double)ds[it][i];
+      result[13][it*m + i] = prob[it][i];
     }
   }
   return result;
